<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor\Type;

class Simple extends AbstractType
{

    /**
     * @return \stdClass
     */
    public function process()
    {
        $this->_item = new \stdClass();

        $this->processBasedata();
        $this->processAssets();
        $this->processAttributes();
        $this->processFilters();
        $this->processSortables();
        $this->processCategories();
        $this->processPrices();
        $this->processInventory();
        return $this->_item;
    }

    protected function processBasedata()
    {
        $this->_item->_id = (int) $this->_product->getId();
        $this->_item->sku = $this->_product->getSku();
        $this->_item->name = $this->_product->getName();

        $this->_item->type = $this->_product->getData('type_id');
        $this->_item->created_at = $this->_product->getData('created_at');
        $this->_item->updated_at = $this->_product->getData('updated_at');

        $this->_item->weight = $this->_product->getData('weight');
        $this->_item->visible = ((int)$this->_product->getData('visibility') == 1) ? "0" : "1";
    }

    protected function processAssets()
    {
        $assets = $this->_product->getData('media_gallery');

        $this->_item->assets = new \stdClass();

        if (count($assets['images'])) {
            foreach ($assets['images'] as $key=>$imageobj) {
                $assets['images'][$key]['file'] = ($imageobj['file']) ? '/catalog/product' . $imageobj['file'] : null;
            }

            $this->_item->assets->images = (count($assets['images'])) ? $assets['images'] : [];
        }

        $this->_item->assets->placeholders = new \stdClass();
        $this->_item->assets->placeholders->image = ($this->_product->getData('image')) ? '/catalog/product' . $this->_product->getData('image') : null;
        $this->_item->assets->placeholders->small_image =  ($this->_product->getData('small_image')) ? '/catalog/product' . $this->_product->getData('small_image') : null;
        $this->_item->assets->placeholders->thumbnail = ($this->_product->getData('thumbnail')) ? '/catalog/product' . $this->_product->getData('thumbnail') : null;
        $this->_item->assets->placeholders->swatch_image = ($this->_product->getData('swatch_image')) ? '/catalog/product' . $this->_product->getData('swatch_image') : null;
    }

    protected function processAttributes()
    {
        $this->_item->attributes = new \stdClass();

        $custom_attributes = $this->_product->getAttributes();

        foreach ($custom_attributes as $attributes) {
            $attribute_code = $attributes->getAttributeCode();

            if (
                !isset($this->_product_attributes[$attributes->getAttributeCode()]) || !$attributes->getIsVisible()) {
                continue;
            }

            if (
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['is_filterable'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['used_for_sort_by'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['is_searchable'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['is_visible_on_front'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['is_filterable_in_search'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['used_in_product_listing'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['used_for_sort_by'] &&
                (int) !$this->_product_attributes[$attributes->getAttributeCode()]['is_visible_in_advanced_search']

            ) {
                continue;
            }

            $this->_item->attributes->{$attribute_code} = new \stdClass();

            $this->_item->attributes->{$attribute_code}->title = $this->_product_attributes[$attributes->getAttributeCode()]['frontend_label'];
            $this->_item->attributes->{$attribute_code}->filterable = (int) $this->_product_attributes[$attributes->getAttributeCode()]['is_filterable'];
            $this->_item->attributes->{$attribute_code}->sortable = (int) $this->_product_attributes[$attributes->getAttributeCode()]['used_for_sort_by'];
            $this->_item->attributes->{$attribute_code}->code = $attribute_code;
            $this->_item->attributes->{$attribute_code}->type = $this->_product_attributes[$attributes->getAttributeCode()]['frontend_input'];

            if (in_array($attributes->getFrontendInput(), $this->allowedInputTypes)) {
                $this->_item->attributes->{$attribute_code}->key = ($this->_product->getData($attributes->getAttributeCode())) ? ($this->_product->getData($attributes->getAttributeCode())) : 0;

                $this->_item->attributes->{$attribute_code}->value =
                    (isset($this->_options[$this->_item->attributes->{$attribute_code}->key]) && $this->_options[$this->_item->attributes->{$attribute_code}->key]["attribute_id"] == $attributes->getAttributeId()) ?
                        $this->_options[$this->_item->attributes->{$attribute_code}->key]["value"] :
                        (string)$this->_product->getAttributeText($attributes->getAttributeCode());
//                $this->_item->attributes->{$attribute_code}->value = ($option_text = $this->_product->getAttributeText($attributes->getAttributeCode())) ? $option_text : "";

                if (
                    isset($this->_product_attributes[$attributes->getAttributeCode()]["additional_data"]) &&
                    isset($this->_swatch_options[$this->_item->attributes->{$attribute_code}->key])
                ) {
                    $additional_object = json_decode($this->_product_attributes[$attributes->getAttributeCode()]["additional_data"]);

                    if (property_exists($additional_object, "swatch_input_type")) {
                        $this->_item->attributes->{$attribute_code}->image = '/attribute/swatch' . $this->_swatch_options[$this->_item->attributes->{$attribute_code}->key]["value"];
                        $this->_item->attributes->{$attribute_code}->type = $additional_object->swatch_input_type;
                    }
                }
            } else {
                $this->_item->attributes->{$attribute_code}->value = $this->_product->getData($attributes->getAttributeCode());
            }
        }
    }

    protected function processFilters()
    {
        $this->_item->filters = new \stdClass();

        foreach ((Array) $this->_item->attributes as $attributes) {
            if ((int) !$attributes->filterable) {
                continue;
            }

            $attribute_code = $attributes->code;

            $this->_item->filters->{$attribute_code} = $attributes;
        }
    }

    protected function processSortables()
    {
        $this->_item->sortables = new \stdClass();

        foreach ((Array) $this->_item->attributes as $attributes) {
            if ((int) !$attributes->sortable) {
                continue;
            }

            $attribute_code = $attributes->code;

            $this->_item->sortables->{$attribute_code} = $attributes;
        }
    }

    protected function processCategories()
    {
        $this->_item->categories = array_map('intval', $this->_product->getCategoryIds());
    }

    protected function processPrices()
    {
        $this->_item->pricing = new \stdClass();
        $this->_item->pricing->base_price = $this->_product->getPrice();
        $this->_item->pricing->final_price = $this->_product->getFinalPrice();

//        foreach ($this->_product->getPriceInfo()->getPrices() as $price) {
//            $this->_item->pricing->{$price->getPriceCode()} = $price->getValue();
//
//        }
    }

    protected function processInventory()
    {
        $this->_item->inventory = new \stdClass();
        $this->_item->inventory->in_stock = $this->_product->isAvailable();
        $this->_item->inventory->backorders = 0;
        $this->_item->inventory->saleable = $this->_product->isSaleable();
    }
}
